==> athena-manager-common-1.0.3-beta-20250821.003840-8-sources.jar: com/digiwin/athena/common/sdk/manager/util/DwSpringHttpUtil.java <==
package com.digiwin.athena.common.sdk.manager.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.digiwin.athena.appcore.auth.AppAuthContextHolder;
import com.digiwin.athena.appcore.auth.domain.AuthoredUser;
import com.digiwin.athena.appcore.exception.BusinessException;
import com.jugg.agile.framework.core.dapper.log.JaLog;
import com.jugg.agile.framework.core.meta.function.JaFunctionP;
import com.jugg.agile.framework.core.util.JaStringUtil;
import com.jugg.agile.spring.util.JaSpringBeanUtil;
import lombok.extern.java.Log;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestClientResponseException;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import static com.digiwin.athena.common.sdk.manager.meta.constants.CommonErrorCode.*;

/**
 * 通用的通过restTemplate 调用外部服务工具类
 *
 * @author gengwei
 * @since 2024/8/19 10:51
 **/
@Slf4j
public class DwSpringHttpUtil {

    private DwSpringHttpUtil() {
    }

    /**
     * get 请求
     */
    public static <T> T get(String url, ParameterizedTypeReference<T> responseType) {
        return get(url, responseType, null, Collections.emptyMap());
    }

    /**
     * get 请求
     */
    public static <T> T get(String url, ParameterizedTypeReference<T> responseType, Map<String, ?> uriVariables) {
        return get(url, responseType, null, uriVariables);
    }

    public static <T> T get(String url, ParameterizedTypeReference<T> responseType, JaFunctionP<HttpHeaders> headerJaFunctionRP, Map<String, ?> uriVariables) {
        HttpHeaders headers = wrapHttpHeaders(headerJaFunctionRP);
        HttpEntity<?> httpEntity = new HttpEntity<>(headers);
        return exchange(url, HttpMethod.GET, httpEntity, responseType, uriVariables);
    }

    /**
     * 发送post请求
     */
    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType) {
        return post(url, body, responseType, null, Collections.emptyMap());
    }

    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType, Map<String, Object> uriVariables) {
        return post(url, body, responseType, null, uriVariables);
    }

    /**
     * 发送post请求
     */
    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType, JaFunctionP<HttpHeaders> headerJaFunctionRP) {
        return post(url, body, responseType, headerJaFunctionRP, Collections.emptyMap());
    }

    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType, JaFunctionP<HttpHeaders> headerJaFunctionRP, Map<String, Object> uriVariables) {
        HttpHeaders headers = wrapHttpHeaders(headerJaFunctionRP);
        HttpEntity<?> httpEntity = new HttpEntity<>(body, headers);
        return exchange(url, HttpMethod.POST, httpEntity, responseType, uriVariables);
    }

    /**
     * 调用spring restTemplate.exchange 方法
     *
     * @author gengwei
     **/
    private static <T> T exchange(String url, HttpMethod httpMethod, HttpEntity<?> requestEntity, Class<T> tClass, Map<String, ?> uriVariables) {
        ResponseEntity<T> respEntity = getRestTemplate().exchange(url, httpMethod, requestEntity, tClass, uriVariables);
        return respEntity.getBody();
    }

    @SuppressWarnings("all")
    private static <T> T exchange(String url, HttpMethod httpMethod, HttpEntity<?> requestEntity, ParameterizedTypeReference<T> responseType, Map<String, ?> uriVariables) {
        ResponseEntity<T> respEntity = null;
        try {
            //临时打印日志
            if (getRestTemplate() == null) {
                log.error("manager-sdk query restTemplate is null");
            }
            respEntity = getRestTemplate().exchange(url, httpMethod, requestEntity, responseType, uriVariables);
        } catch (ResourceAccessException ex) {
            //httpStatus!=200
            //访问其他组件时网络异常，请稍后重试 封装成BusinessException异常抛出
            throw BusinessException.buildErrorCode(P_AI_500_5004);
        } catch (RestClientResponseException ex) {
            //400 500 错误  可以获取到responseBodyAsString进行解析获取errorCode和errorMessage进行透传 封装成BusinessException异常抛出
            parseRestClientResponseException(url, ex);
        } catch (RestClientException ex) {
            //响应体解析错误
            if (ex.getCause() instanceof HttpMessageConversionException) {
                throw BusinessException.buildErrorCode(P_AI_500_5006);
            }
            throw ex;
        }
        return respEntity.getBody();
    }


    private static HttpHeaders wrapHttpHeaders(JaFunctionP<HttpHeaders> headerJaFunctionRP) {
        HttpHeaders headers = initHttpHeaders();
        if (headerJaFunctionRP != null) {
            headerJaFunctionRP.apply(headers);
        }
        return headers;
    }

    private static HttpHeaders initHttpHeaders() {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        AuthoredUser authoredUser = AppAuthContextHolder.getContext().getAuthoredUser();
        if (authoredUser != null) {
            headers.add("token", authoredUser.getToken());
        }

        headers.add("locale", LocaleContextHolder.getLocale().toString());
        return headers;
    }

    private static void parseRestClientResponseException(String url, RestClientResponseException ex) {
        JSONObject errorException = null;
        try {
            errorException = JSON.parseObject(ex.getResponseBodyAsString(), JSONObject.class);
        } catch (Exception e) {
            //如果转换异常肯定是404 502 503 504等nginx抛出的异常 这里打印日志方便定位
            JaLog.error("{} error : {}", url, e);
        }
        if (null == errorException) {
            throw BusinessException.buildErrorCode(P_AI_500_5000);
        }
        String errorCode = getErrorCode(errorException);
        String errorMessage = getErrorMessage(errorException);
        throw BusinessException.create(errorCode, errorMessage).buildLog(errorMessage);
    }

    private static String getErrorCode(JSONObject errorException) {
        return Optional.ofNullable(errorException.getString("errorCode"))
                .filter(code -> !JaStringUtil.isEmpty(code))
                .orElse(P_AI_500_5000);
    }

    private static String getErrorMessage(JSONObject errorException) {
        return errorException.getString("errorMessage");
    }

    private static RestTemplate restTemplate = null;

    public static RestTemplate getRestTemplate() {
        if (Objects.isNull(restTemplate)) {
            synchronized (DwSpringHttpUtil.class) {
                if (Objects.isNull(restTemplate)) {
                    restTemplate = JaSpringBeanUtil.getBean(RestTemplate.class);
                }
            }
        }
        return restTemplate;
    }
}

==> athena-manager-common-2.6.13.0-beta-20250819.065425-6-sources.jar: com/digiwin/athena/common/sdk/manager/util/DwSpringHttpUtil.java <==
package com.digiwin.athena.common.sdk.manager.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.digiwin.athena.appcore.auth.AppAuthContextHolder;
import com.digiwin.athena.appcore.auth.domain.AuthoredUser;
import com.digiwin.athena.appcore.exception.BusinessException;
import com.jugg.agile.framework.core.dapper.log.JaLog;
import com.jugg.agile.framework.core.meta.function.JaFunctionP;
import com.jugg.agile.framework.core.util.JaStringUtil;
import com.jugg.agile.spring.util.JaSpringBeanUtil;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageConversionException;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestClientResponseException;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.Map;
import java.util.Optional;

import static com.digiwin.athena.common.sdk.manager.meta.constants.CommonErrorCode.*;

/**
 * 通用的通过restTemplate 调用外部服务工具类
 *
 * @author gengwei
 * @since 2024/8/19 10:51
 **/
public class DwSpringHttpUtil {

    private DwSpringHttpUtil() {
    }

    /**
     * get 请求
     */
    public static <T> T get(String url, ParameterizedTypeReference<T> responseType) {
        return get(url, responseType, null, Collections.emptyMap());
    }

    /**
     * get 请求
     */
    public static <T> T get(String url, ParameterizedTypeReference<T> responseType, Map<String, ?> uriVariables) {
        return get(url, responseType, null, uriVariables);
    }

    public static <T> T get(String url, ParameterizedTypeReference<T> responseType, JaFunctionP<HttpHeaders> headerJaFunctionRP, Map<String, ?> uriVariables) {
        HttpHeaders headers = wrapHttpHeaders(headerJaFunctionRP);
        HttpEntity<?> httpEntity = new HttpEntity<>(headers);
        return exchange(url, HttpMethod.GET, httpEntity, responseType, uriVariables);
    }

    /**
     * 发送post请求
     */
    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType) {
        return post(url, body, responseType, null, Collections.emptyMap());
    }

    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType, Map<String, Object> uriVariables) {
        return post(url, body, responseType, null, uriVariables);
    }

    /**
     * 发送post请求
     */
    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType, JaFunctionP<HttpHeaders> headerJaFunctionRP) {
        return post(url, body, responseType, headerJaFunctionRP, Collections.emptyMap());
    }

    public static <T> T post(String url, Object body, ParameterizedTypeReference<T> responseType, JaFunctionP<HttpHeaders> headerJaFunctionRP, Map<String, Object> uriVariables) {
        HttpHeaders headers = wrapHttpHeaders(headerJaFunctionRP);
        HttpEntity<?> httpEntity = new HttpEntity<>(body, headers);
        return exchange(url, HttpMethod.POST, httpEntity, responseType, uriVariables);
    }

    /**
     * 调用spring restTemplate.exchange 方法
     *
     * @author gengwei
     **/
    private static <T> T exchange(String url, HttpMethod httpMethod, HttpEntity<?> requestEntity, Class<T> tClass, Map<String, ?> uriVariables) {
        ResponseEntity<T> respEntity = Instance.restTemplate.exchange(url, httpMethod, requestEntity, tClass, uriVariables);
        return respEntity.getBody();
    }

    @SuppressWarnings("all")
    private static <T> T exchange(String url, HttpMethod httpMethod, HttpEntity<?> requestEntity, ParameterizedTypeReference<T> responseType, Map<String, ?> uriVariables) {
        ResponseEntity<T> respEntity = null;
        try {
            respEntity = Instance.restTemplate.exchange(url, httpMethod, requestEntity, responseType, uriVariables);
        } catch (ResourceAccessException ex) {
            //httpStatus!=200
            //访问其他组件时网络异常，请稍后重试 封装成BusinessException异常抛出
            throw BusinessException.buildErrorCode(P_AI_500_5004);
        } catch (RestClientResponseException ex) {
            //400 500 错误  可以获取到responseBodyAsString进行解析获取errorCode和errorMessage进行透传 封装成BusinessException异常抛出
            parseRestClientResponseException(url, ex);
        } catch (RestClientException ex) {
            //响应体解析错误
            if (ex.getCause() instanceof HttpMessageConversionException) {
                throw BusinessException.buildErrorCode(P_AI_500_5006);
            }
            throw ex;
        }
        return respEntity.getBody();
    }


    private static HttpHeaders wrapHttpHeaders(JaFunctionP<HttpHeaders> headerJaFunctionRP) {
        HttpHeaders headers = initHttpHeaders();
        if (headerJaFunctionRP != null) {
            headerJaFunctionRP.apply(headers);
        }
        return headers;
    }

    private static HttpHeaders initHttpHeaders() {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        AuthoredUser authoredUser = AppAuthContextHolder.getContext().getAuthoredUser();
        if (authoredUser != null) {
            headers.add("token", authoredUser.getToken());
        }

        headers.add("locale", LocaleContextHolder.getLocale().toString());
        return headers;
    }

    private static class Instance {
        private static final RestTemplate restTemplate = JaSpringBeanUtil.getBean(RestTemplate.class);
    }

    private static void parseRestClientResponseException(String url, RestClientResponseException ex) {
        JSONObject errorException = null;
        try {
            errorException = JSON.parseObject(ex.getResponseBodyAsString(), JSONObject.class);
        } catch (Exception e) {
            //如果转换异常肯定是404 502 503 504等nginx抛出的异常 这里打印日志方便定位
            JaLog.error("{} error : {}", url, e);
        }
        if (null == errorException) {
            throw BusinessException.buildErrorCode(P_AI_500_5000);
        }
        String errorCode = getErrorCode(errorException);
        String errorMessage = getErrorMessage(errorException);
        throw BusinessException.create(errorCode, errorMessage).buildLog(errorMessage);
    }

    private static String getErrorCode(JSONObject errorException) {
        return Optional.ofNullable(errorException.getString("errorCode"))
                .filter(code -> !JaStringUtil.isEmpty(code))
                .orElse(P_AI_500_5000);
    }

    private static String getErrorMessage(JSONObject errorException) {
        return errorException.getString("errorMessage");
    }
}
